﻿using System;
using System.IO;
using System.Linq;
using AsyncPrint.MvvmStuff;
using Telerik.Windows.Controls;
using Telerik.Windows.Documents.Fixed.FormatProviders;
using Telerik.Windows.Documents.Fixed.FormatProviders.Pdf;
using Telerik.Windows.Documents.Fixed.Print;
using System.Windows.Controls;
using System.Threading;
using System.Windows;
using Telerik.Windows.Documents.Fixed.Model;
using System.Windows.Threading;
using System.Windows.Xps.Packaging;
using System.IO.Packaging;
using System.Windows.Xps;
using System.Printing;
using System.Windows.Documents;
using System.Threading.Tasks;
using Microsoft.Win32;
using AsyncPrint.Extensions;
using AsyncPrint.Objects;

namespace AsyncPrint
{
    class MainWindowViewModel : BindableBase
    {
        private string timeStamp;
        private string selectedFile;
        public Window MyWindow { get; set; }
        public RelayCommand PrintDocumentAsyncToDefaultCommand { get; private set; }
        public RelayCommand PrintDocumentAsyncWithPrintDialogCommand { get; private set; }
        public RelayCommand UpdateLabelCommand { get; private set; }
        public RelayCommand SelectFileDialogCommand { get; private set; }

        public string SelectedFile 
        {
            get { return selectedFile; }
            set { SetProperty(ref selectedFile, value); }
        }

        public string TimeStamp 
        {
            get { return timeStamp; }
            set { SetProperty(ref timeStamp, value); }
        }

        public MainWindowViewModel()
        {
            PrintDocumentAsyncToDefaultCommand = new RelayCommand(OnPrintDocumentAsyncToDefault);
            PrintDocumentAsyncWithPrintDialogCommand = new RelayCommand(OnPrintDocumentAsyncWithPrintDialog);
            UpdateLabelCommand = new RelayCommand(OnUpdateLabel);
            SelectFileDialogCommand = new RelayCommand(OnSelectFileDialog);
            TimeStamp = DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss.fff");
            SelectedFile = SavedDocument();
        }

        private void OnPrintDocumentAsyncWithPrintDialog()
        {
            string savedPdfLocation = SavedDocument();
            PrintAsyncWithPrintDialog(savedPdfLocation);
        }        

        private void OnPrintDocumentAsyncToDefault()
        {
            string savedPdfLocation = SavedDocument();
            PrintAsyncToDefaultPrinter(savedPdfLocation);
        }

        private void OnSelectFileDialog()
        {
            OpenFileDialog openFileDialog = new OpenFileDialog();
            if (openFileDialog.ShowDialog() == true)
                SelectedFile = openFileDialog.FileName;
        }

        private string SavedDocument()
        {
            // This code uses PdfSharp (on nuget) to create a .pdf file on disk and returns the path of the saved pdf.
            // This code has room for async as well.

            return !string.IsNullOrEmpty(SelectedFile) ? SelectedFile 
                : Path.Combine(System.AppDomain.CurrentDomain.BaseDirectory, 
                                "Documents", 
                                "Telerik_Test_Studio_Quick_Start_Guide.pdf");
        }

        // Good, but prints silently to the default printer
        private void PrintAsyncToDefaultPrinter(string fileLocation)
        {
            var thread = new Thread((parameter) =>
            {
                PrintDocument(fileLocation, new PrintDialog());
                Thread.CurrentThread.Abort();
            });

            thread.SetApartmentState(ApartmentState.STA);
            thread.Start();
        }

        // Final Working Solution
        private void PrintAsyncWithPrintDialog(string fileLocation)
        {
            PrintDialog printDialog = new PrintDialog();

            // Exit out of here if the user doesn't click "print" 
            if (printDialog.ShowDialog() == false) { return; };

            // Extract the pieces we want/need from the Print Dialog (note that we CANNOT do this inside the new thread, we have to do it outside!)
            string printerFullName = printDialog.PrintQueue.FullName;
            int copyCount = printDialog.PrintTicket.CopyCount ?? 0;

            var thread = new Thread(() =>
            {
                // Now that we're inside this thread, we need to make a new PrintDialog and set its fresh properties to the same as the user populated one
                PrintDialog pd = new PrintDialog()
                {
                    PrintQueue = new PrintQueue(new PrintServer(), printerFullName),
                    PrintTicket = new PrintTicket() { CopyCount = copyCount }
                };

                // Print the document and close the thread when done
                PrintDocument(fileLocation, pd);
                Thread.CurrentThread.Abort();
            });

            thread.SetApartmentState(ApartmentState.STA);
            thread.Start();
        }

        private static void PrintDocument(string fileLocation, PrintDialog dialog)
        {
            RadPdfViewer pdfViewer = new RadPdfViewer();
            PdfFormatProvider provider;

            using (MemoryStream memoryStream = new MemoryStream())
            {
                using (Stream stream = File.OpenRead(fileLocation))
                {
                    stream.CopyTo(memoryStream);
                }
                
                provider = new PdfFormatProvider(memoryStream, FormatProviderSettings.ReadOnDemand);
                pdfViewer.Document = provider.Import();

                string fileName = new FileInfo(fileLocation).Name;
                PrintSettings printSettings = new PrintSettings() { DocumentName = fileName };

                pdfViewer.Print(dialog, printSettings);
            }
        }

        private void OnUpdateLabel()
        {
            TimeStamp = DateTime.Now.ToString("MM/dd/yyyy hh:mm:ss.fff");
        }
    }
}
