﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;

namespace AsyncPrint.MvvmStuff
{
    /// <summary>
    /// The base class for all View Models and is responsible for encapsulating the INPC implementation and anything else common to all view models.
    /// </summary>
    public abstract class BindableBase : INotifyPropertyChanged
    {
        /// <summary>
        /// This event triggers when a property value changes.
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged = delegate { };

        /// <summary>
        /// Checks to see if the property being changed is different than what's already there, if it's different
        /// it'll set the property and notify all other listeners (only when necessary).
        /// This method is intended to be invoked from a "Set" block of any property that needs INPC. 
        /// </summary>
        protected virtual void SetProperty<T>(ref T member, T val, [CallerMemberName] string propertyName = null)
        {
            // If the property changed, set it and trigger the PropertyChanged event
            if (object.Equals(member, val)) { return; }

            member = val;
            PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        /// <summary>
        /// This method will notify all listeners that a property value has changed.
        /// </summary>
        protected virtual void OnPropertyChanged(string propertyName)
        {
            PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
